@@ -19,6 +19,7 @@ from requests.packages.urllib3.exceptions import InsecureRequestWarning
 # sys.path.insert(0, os.path.join(os.path.dirname(os.path.abspath(__file__))))
 
 if server_type == 'feeds':
+    from gracie_feeds_api.GracieAuth import GracieSSOAuth
     from gracie_feeds_api.GracieAPIClassLoader import GracieApiClassLoader
     from gracie_feeds_api.GracieErrors import GracieAPIResponseError
     from gracie_feeds_api.GracieErrors import GracieAuthError
@@ -26,7 +27,9 @@ if server_type == 'feeds':
     from gracie_feeds_api.GracieErrors import MissingDirectory
     from gracie_feeds_api.GracieErrors import FileAccessError
     from gracie_feeds_api.GracieErrors import gracieAPIUnknownParameter
+    from gracie_feeds_api.GracieErrors import gracieAPIUnknownSendDataType
 elif server_type == 'dictionary':
+    from gracie_dictionary_api.GracieAuth import GracieSSOAuth
     from gracie_dictionary_api.GracieAPIClassLoader import GracieApiClassLoader
     from gracie_dictionary_api.GracieErrors import GracieAPIResponseError
     from gracie_dictionary_api.GracieErrors import GracieAuthError
@@ -47,7 +50,7 @@ else:
     CONTROLLERS_CLASSES_PATH = server_type + '_controllers_classes'
 
 
-class GracieBaseAPI(GracieAPIResponseError, GracieError, GracieAuthError):
+class GracieBaseAPI(GracieAPIResponseError, GracieError, GracieAuthError, GracieSSOAuth):
     _session = None
 
     def __init__(self):
@@ -72,10 +75,11 @@ class GracieBaseAPI(GracieAPIResponseError, GracieError, GracieAuthError):
                             _v_n = v_n
 
                         if v_n in parameters_names:
-                            if parameters_names[v_n]['in'] == 'query':
+                            if parameters_names[v_n]['in'] == 'query' or \
+                                    parameters_names[v_n]['in'] == 'formData':
                                 params.update({v_n: set_api_params[var_name][v_n]})
-                            elif parameters_names[var_name][v_n] == 'body':
-                                data.update({v_n: set_api_params[var_name][v_n]})
+                            elif parameters_names[v_n]['in'] == 'body':
+                                data = set_api_params[var_name][v_n]
                             else:
                                 raise gracieAPIUnknownSendDataType(
                                     '%s is not query or body' % set_api_params[var_name]['in'])
@@ -92,8 +96,8 @@ class GracieBaseAPI(GracieAPIResponseError, GracieError, GracieAuthError):
                         var_name = parameters_names_map[var_name]
 
                     if var_name in parameters_names:
-                        if parameters_names[var_name]['in'] == 'query' or parameters_names[var_name][
-                            'in'] == 'formData':
+                        if parameters_names[var_name]['in'] == 'query' or \
+                                parameters_names[var_name]['in'] == 'formData':
                             params.update({var_name: set_api_params[var_name]})
                         elif parameters_names[var_name]['in'] == 'body':
                             data = set_api_params[var_name]
@@ -109,14 +113,8 @@ class GracieBaseAPI(GracieAPIResponseError, GracieError, GracieAuthError):
     def _start_session(self):
         if self._session:
             self._close_session()
-
         self._session = requests.session()
 
-        login_data = self._controller_classes['loginController'].login(self, login=self._login_name,
-                                                                       password=self._login_password)
-        if 'status' in login_data and login_data['status'] is False:
-            raise BaseException(login_data['message'])
-
     def _close_session(self):
         self._session = None
 
@@ -153,7 +151,7 @@ class GracieBaseAPI(GracieAPIResponseError, GracieError, GracieAuthError):
 
         return hash_md5.hexdigest()
 
-    def _process_api(self, controller_name, api, actions, params, data):
+    def _process_api(self, controller_name, api, actions, params, data, consumes=None):
         if not self._session:
             self._start_session()
 
@@ -166,7 +164,7 @@ class GracieBaseAPI(GracieAPIResponseError, GracieError, GracieAuthError):
                 files = self._prepare_files_for_upload(params['files'])
                 del params['files']
 
-        api_url = '%s://%s:%s%s' % (self._http_protocol, self._host, self._port, api)
+        api_url = '%s://%s:%s%s' % (self._http_protocol, self._gracie_host, self._port, api)
 
         if 'post' in actions:
             session_action = self._session.post
@@ -179,18 +177,23 @@ class GracieBaseAPI(GracieAPIResponseError, GracieError, GracieAuthError):
         else:
             raise GracieError('Bad http action %s attempted on api: %s' % (','.join(actions), api))
 
+        headers = self.access_headers
         if data:
-            data = data.encode('utf-8')
-            headers = {'Content-type': 'text/plain; charset=utf-8'}
-        else:
-            headers = None
+            if not consumes:
+                data = data.encode('utf-8')
+                headers.update({'Content-type': 'text/plain; charset=utf-8'})
+            else:
+                # only serialize list, dict, and tuple - other types are fine
+                if isinstance(data, list) or isinstance(data, dict) or isinstance(data, tuple):
+                    data = json.dumps(data)
+                headers.update({'Content-type': '%s' % consumes[0]})
 
         if not files:
             response = session_action(api_url, params=params, data=data, verify=self._verify_ssl, proxies=self._proxies,
                                       headers=headers)
         else:
             response = session_action(api_url, params=params, data=data, files=files, verify=self._verify_ssl,
-                                      proxies=self._proxies)
+                                      proxies=self._proxies, headers=headers)
 
         if response.status_code == 401:
             if controller_name == 'logoutController':
@@ -246,9 +249,7 @@ class GracieAPI(GracieApiClassLoader, GracieBaseAPI):
         full_results: Return full results from API calls, not just results from search
     """
 
-    _login_name = None
-    _login_password = None
-    _host = None
+    _gracie_host = None
     _port = 443
     _verify_ssl = False
     _http_protocol = 'https'
@@ -263,16 +264,16 @@ class GracieAPI(GracieApiClassLoader, GracieBaseAPI):
     _remove_task_data_on_complete = True
     _full_results = False
 
-    def __init__(self, host, port, login_name, login_password, require_ssl=True, verify_ssl=False,
+    def __init__(self, host, sso_host, port, login_name, login_password,
+                 require_ssl=True,
+                 verify_ssl=False,
                  wait_for_task_complete=True,
                  remove_task_data_on_complete=True,
                  proxies=None,
                  download_file_location=None):
 
-        self._host = host
+        self._gracie_host = host
         self._port = port
-        self._login_name = login_name
-        self._login_password = login_password
         self._wait_for_task_complete = wait_for_task_complete
         self._remove_task_data_on_complete = remove_task_data_on_complete
         self._proxies = proxies
@@ -293,6 +294,7 @@ class GracieAPI(GracieApiClassLoader, GracieBaseAPI):
 
         GracieBaseAPI.__init__(self)
         GracieApiClassLoader.__init__(self, CONTROLLERS_CLASSES_PATH)
+        GracieSSOAuth.__init__(self, sso_host, login_name, login_password)
 
     @property
     def wait_for_task_complete(self):
@@ -359,18 +361,32 @@ class GracieAPI(GracieApiClassLoader, GracieBaseAPI):
     def _wait_for_task_results(self, task_data, remove_results=True, sleep_time=1):
         while True:
             if task_data and 'status' in task_data:
-                task_data = self.call_api('tasksController', 'retrieve', id=task_data['id'])
+
+                # todo - yuck! please update backend APIs!
+                if server_type == 'feeds' and 'taskId' in task_data:
+                    task_id = task_data['taskId']
+                elif server_type == 'dictionary' and 'id' in task_data:
+                    task_id = task_data['id']
+                else:
+                    raise GracieAPIResponseError('Task id not found in results')
+
+                task_data = self.call_api('tasksController', 'retrieve', id=task_id)
                 if task_data and 'status' in task_data and \
                         type(task_data['status']) is str and \
                         (task_data['status'].lower() == 'completed' or task_data['status'].lower() == 'failed'):
                     break
             time.sleep(sleep_time)
 
+        # todo - yuck! please update backend APIs!
         if remove_results:
-            self.call_api('tasksController', 'delete', id=task_data['id'])
-
-        if task_data['error']:
-            raise GracieError(task_data['error'])
+            if server_type == 'dictionary':
+                self.call_api('tasksController', 'delete', id=task_id)
+                if task_data['error']:
+                    raise GracieError(task_data['error'])
+            else:
+                self.call_api('tasksController', 'remove', id=task_id)
+                if task_data['errorMessage']:
+                    raise GracieError(task_data['errorMessage'])
 
         if self._full_results:
             return task_data
@@ -404,26 +420,27 @@ class GracieAPI(GracieApiClassLoader, GracieBaseAPI):
                     found = True
                 if not found and p != 'self' and p != '**kwargs':
                     function_params.remove('self')
-                    raise GracieError('Missing API parameter(s) %s' % ', '.join(function_params),
+                    raise GracieError('Missing API parameter(s) "%s" for %s' % (', '.join(function_params),
+                                                                                self._controller_name),
                                       'Required parameter: %s' % ', '.join(function_params))
 
         if _kwargs and len(_kwargs) > 0:
             kwargs = _kwargs
 
-        counter = 0
+        # counter = 0
         while True:
-            counter += 1
-            try:
-                api_ret_data = eval(function_sig)
-                if controller_name != 'tasksController' and 'status' in api_ret_data and self._wait_for_task_complete:
-                    return self._wait_for_task_results(api_ret_data, self._remove_task_data_on_complete)
-                return api_ret_data
+            # counter += 1
+            # try:
+            api_ret_data = eval(function_sig)
+            if controller_name != 'tasksController' and \
+                    'status' in api_ret_data and \
+                    api_ret_data['status'].lower() != 'unchanged' and \
+                    self._wait_for_task_complete:
+                return self._wait_for_task_results(api_ret_data, self._remove_task_data_on_complete)
+            return api_ret_data
 
-            except GracieAPIResponseError as e:
-                if counter > 2:
-                    raise GracieError('API error: (%s) %s' % (e.parameter, e.message))
-
-                if e.message == 'Invalid session':
-                    self._session = None
-                else:
-                    break
+            # except GracieAPIResponseError as e:
+            #     if counter > 2 or e.message == 401:
+            #         raise GracieError('API error: (%s) %s' % (e.parameter, ' - '.join(e.args)))
+            #     else:
+            #         break
