@@ -13,8 +13,9 @@ class enginesClusterController(GracieBaseAPI):
         parameters_names_map = {}
         api = '/engines/cluster/status'
         actions = ['post']
+        consumes = ['application/json']
         params, data = self._format_params_for_api(locals(), all_api_parameters, parameters_names_map)
-        return self._process_api(self._controller_name, api, actions, params, data)
+        return self._process_api(self._controller_name, api, actions, params, data, consumes)
 
     def clusterWorkers(self):
         """"""
@@ -23,5 +24,6 @@ class enginesClusterController(GracieBaseAPI):
         parameters_names_map = {}
         api = '/engines/cluster/workers'
         actions = ['post']
+        consumes = ['application/json']
         params, data = self._format_params_for_api(locals(), all_api_parameters, parameters_names_map)
-        return self._process_api(self._controller_name, api, actions, params, data)
+        return self._process_api(self._controller_name, api, actions, params, data, consumes)
