@@ -14,6 +14,9 @@ class summarizerController(GracieBaseAPI):
             maxSentences: (integer): maxSentences
             text: (type): text
 
+        Consumes:
+            application/json
+
         Returns:
             application/json;charset=UTF-8
         """
@@ -22,5 +25,6 @@ class summarizerController(GracieBaseAPI):
         parameters_names_map = {}
         api = '/summarize'
         actions = ['post']
+        consumes = ['application/json']
         params, data = self._format_params_for_api(locals(), all_api_parameters, parameters_names_map)
-        return self._process_api(self._controller_name, api, actions, params, data)
+        return self._process_api(self._controller_name, api, actions, params, data, consumes)
